@@ -15,6 +15,8 @@ Higher memory usage of ACore functions can increase the thread switching time dr
     Keep in mind, that all the ACore classes run in a different thread, called by the sounddevice callback function.
     Therfore reading or writing to variables, except for initialization, has to be thread safe!
 """
+# classes and functions in here should never reference big classes like Interface, Processors, IO, ...
+# Ensure that those are never imported!!!
 import queue
 from abc import ABC, abstractmethod
 from typing import TYPE_CHECKING, Optional, Tuple
@@ -23,7 +25,6 @@ import numpy as np
 import sounddevice as sd
 
 if TYPE_CHECKING:
-    # classes and functions in here should never reference big classes like Interface etc...
     from .typing import ABlock, ABuffer, CData, FFTInput, InAs
 
 
@@ -42,6 +43,7 @@ class AGenerator(ABC):
         self._out_buffer = out_buffer
         self._blocksize = blocksize
         self._frame = start_frame
+        self._start_frame = start_frame
 
         # set output channels and update _out_buf
         self._out_buf: "ABuffer"
@@ -57,7 +59,7 @@ class AGenerator(ABC):
         self._out_chs = value
         # update _out_buf size
         if self._out_buffer and value is not None:
-            self._out_buf = np.empty((self._blocksize, value), dtype=np.float32)
+            self._out_buf = np.zeros((self._blocksize, value), dtype=np.float32)
 
     def upstream(self, outdata: "ABlock", ch: int, frame: int) -> None:
         """This method is called by other AProcessors, connected to the outputs,
@@ -72,8 +74,12 @@ class AGenerator(ABC):
             ch: The channel corresponding to the outdata at hand (zero indexed).
             frame: Current frame number to be processed, this always increases by +1 and is incremented by the callback.
         """
+        # if smaller than start frame, reset the generator
+        if frame < self._start_frame:
+            self._reset()
+            return
         # if the next frame is called, increment and set buffer reload flag
-        if frame != self._frame:
+        elif frame != self._frame:
             self._frame = frame
             self._inc()
             self._reload = True
@@ -102,6 +108,7 @@ class AGenerator(ABC):
             outdata: The 1D array to write to, with length blocksize.
             ch: The channel corresponding to the outdata at hand (zero indexed).
         """
+        return None
 
     @abstractmethod
     def _inc(self) -> None:
@@ -109,6 +116,14 @@ class AGenerator(ABC):
         If the class changes over time, this function can be used to perform these changes.
         It is called exactly once after all channels of the class have been processed.
         """
+        return None
+
+    @abstractmethod
+    def _reset(self) -> None:
+        """This function is envoked by `upstream()`.
+        It's called when the first frame is requested,
+        and should reset the generator into initial state."""
+        self._frame = self._start_frame
 
 
 class AEffect(ABC):
@@ -129,6 +144,7 @@ class AEffect(ABC):
         self._out_buffer = out_buffer
         self._blocksize = blocksize
         self._frame = start_frame
+        self._start_frame = start_frame
 
         # set in-/output channels and update buffers
         self._in_buf: "ABuffer"
@@ -149,7 +165,7 @@ class AEffect(ABC):
         self._in_as = value
         # update _in_buf size
         if self._in_buffer and value is not None:
-            self._in_buf = np.empty((self._blocksize, len(value)), dtype=np.float32)
+            self._in_buf = np.zeros((self._blocksize, len(value)), dtype=np.float32)
 
     @property
     def out_chs(self) -> int:
@@ -160,7 +176,7 @@ class AEffect(ABC):
         self._out_chs = value
         # update _out_buf size
         if self._out_buffer and value is not None:
-            self._out_buf = np.empty((self._blocksize, value), dtype=np.float32)
+            self._out_buf = np.zeros((self._blocksize, value), dtype=np.float32)
 
     def upstream(self, outdata: "ABlock", ch: int, frame: int) -> None:
         """This method is called by other AProcessors, connected to the outputs,
@@ -175,8 +191,12 @@ class AEffect(ABC):
             ch: The channel corresponding to the outdata at hand (zero indexed).
             frame: Current frame number to be processed, this always increases by +1 and is incremented by the callback.
         """
+        # if smaller than start frame, reset the generator
+        if frame < self._start_frame:
+            self._reset()
+            return
         # if the next frame is called, increment and set buffer reload flag
-        if frame != self._frame:
+        elif frame != self._frame:
             self._frame = frame
             self._inc()
             self._reload = True
@@ -238,9 +258,10 @@ class AEffect(ABC):
 
         Example: Input buffer example
             ```python
-                if self._in_as[ch][0] is not None:
+                in_a = self._in_as[ch][0]
+                if in_a is not None:
                     # get outdata from upstream
-                    self._in_as[ch][0].upstream(outdata, self._in_as[ch][1], frame)
+                    in_a.upstream(outdata, self._in_as[ch][1], self._frame)
                     # process outdata here
                     ...
             ```
@@ -259,6 +280,14 @@ class AEffect(ABC):
         If the class changes over time, this function can be used to perform these changes.
         It is called exactly once per audio frame, but not for the first one.
         """
+        return None
+
+    @abstractmethod
+    def _reset(self) -> None:
+        """This function is envoked by `upstream()`.
+        It's called when the first frame is requested,
+        and should reset the effect into initial state."""
+        self._frame = self._start_frame
 
 
 class AAnalyzer(ABC):
@@ -275,7 +304,7 @@ class AAnalyzer(ABC):
         """
         self._in_buffer = in_buffer
         self._blocksize = blocksize
-        self._frame = start_frame
+        self._start_frame = start_frame
 
         # set input channels and update _in_buf
         self._in_buf: "ABuffer"
@@ -302,9 +331,14 @@ class AAnalyzer(ABC):
             self._update_in_buf(len(value))
 
     def _update_in_buf(self, channels: int) -> None:
-        self._in_buf = np.empty((self.buffersize, channels), dtype=np.float32)
+        self._in_buf = np.zeros((self.buffersize, channels), dtype=np.float32)
 
     def start_upstream(self, frame: int) -> None:
+        # no additional frame processing necessary,
+        # because this function is only called once per frame
+        if frame < self._start_frame:
+            self._reset()
+            return
         # if in_buffer is enabled fill _in_buf
         if self._in_buffer:
             self._fill_in_buf(frame)
@@ -339,11 +373,18 @@ class AAnalyzer(ABC):
             NUMPY_ARRAY_TO_WRITE_TO.fill(0)
             for in_ch, in_a in enumerate(self._in_as):
                 if in_a[0] is not None:
-                    in_a[0].upstream(NUMPY_ARRAY_TO_WRITE_TO[:, in_ch], in_a[1], frame)
+                    in_a[0].upstream(NUMPY_ARRAY_TO_WRITE_TO[:, in_ch], in_a[1], self._frame)
             ```
         """
         raise NotImplementedError("For disabled self._in_buffer, you must override this function.")
 
+    @abstractmethod
+    def _reset(self) -> None:
+        """This function is envoked by `upstream()`.
+        It's called when the first frame is requested,
+        and should reset the analyzer into initial state."""
+        return None
+
 
 class AAnalyzerBuf(AAnalyzer):
     def __init__(self, bufsize: int, blocksize: int, start_frame: int) -> None:
@@ -391,6 +432,12 @@ class AAnalyzerBuf(AAnalyzer):
                 self._in_buf[:rest, :] = self._in_buf[self._bufsize:self._in_buf_idx]
             self._in_buf_idx = rest
 
+    def _reset(self) -> None:
+        self._in_buf_idx = 0
+        while not self.buf_queue.empty():
+            self.buf_queue.get(block=False)
+        return super()._reset()
+
 
 class AInterface:
     def __init__(self, blocksize: int, start_frame: int):
@@ -402,10 +449,12 @@ class AInterface:
             start_frame: The number of the first frame (to start counting from).
         """
         self._out_buf: "ABuffer"
-        self.out_chs = None
+        self.out_ch_map = ()
+        self.in_ch_map = ()
 
         self._blocksize = blocksize
         self._frame = start_frame
+        self.ctime: Optional["CData"] = None
 
         self.end_frame: Optional[int] = None
 
@@ -421,15 +470,23 @@ class AInterface:
         self._in_as = value
 
     @property
-    def out_chs(self) -> Optional[int]:
-        return self._out_chs
+    def out_ch_map(self) -> Tuple[int, ...]:
+        return self._out_ch_map
 
-    @out_chs.setter
-    def out_chs(self, value: Optional[int]) -> None:
-        self._out_chs = value
-        if value is not None and value > 0:
+    @out_ch_map.setter
+    def out_ch_map(self, value: Tuple[int, ...]) -> None:
+        self._out_ch_map = value
+        if value:
             # update _out_buf size
-            self._out_buf = np.empty((self._blocksize, value), dtype=np.float32)
+            self._out_buf = np.zeros((self._blocksize, len(value)), dtype=np.float32)
+
+    @property
+    def in_ch_map(self) -> Tuple[int, ...]:
+        return self._in_ch_map
+
+    @in_ch_map.setter
+    def in_ch_map(self, value: Tuple[int, ...]) -> None:
+        self._in_ch_map = value
 
     @property
     def alzs(self) -> Tuple["AAnalyzer", ...]:
@@ -439,25 +496,34 @@ class AInterface:
     def alzs(self, value: Tuple["AAnalyzer", ...]) -> None:
         self._alzs = value
 
+    @property
+    def frame(self) -> int:
+        return self._frame
+
+    @frame.setter
+    def frame(self, value: int) -> None:
+        self._frame = value
+
     def callback(self,
                  indata: "ABuffer",
                  outdata: "ABuffer",
                  frames: int,
                  ctime: "CData",
                  status: sd.CallbackFlags) -> None:
-        if self._frame >= 0:
-            # copy indata so it can be processed by upstream()
-            if self.out_chs:
-                self._out_buf[:] = indata
-            # call upstream method of the outputs connected to the inputs
-            if self.in_as != ():
-                outdata.fill(0)
-                for in_ch, in_a in enumerate(self._in_as):
-                    if in_a[0] is not None:
-                        in_a[0].upstream(outdata[:, in_ch], in_a[1], self._frame)
-            # call AAnalyzers start_upstream method (because they wont get called otherwise)
-            for alz in self._alzs:
-                alz.start_upstream(self._frame)
+        if self.ctime is None:
+            self.ctime = ctime
+        # copy indata so it can be processed by upstream()
+        if self.out_ch_map:
+            self._out_buf[:] = indata[:, self.out_ch_map]
+        # call upstream method of the outputs connected to the inputs
+        if self.in_as != ():
+            outdata.fill(0)
+            for in_ch, in_a in enumerate(self._in_as):
+                if in_a[0] is not None:
+                    in_a[0].upstream(outdata[:, self.in_ch_map[in_ch]], in_a[1], self._frame)
+        # call AAnalyzers start_upstream method (because they wont get called otherwise)
+        for alz in self._alzs:
+            alz.start_upstream(self._frame)
         self._frame += 1  # Overflow?
         if self.end_frame is not None and self.end_frame <= self._frame:
             raise sd.CallbackStop
