@@ -1,9 +1,11 @@
 """In this submodule you can find all generators, so "audio processors" with one or multiple outputs."""
+import logging
 import queue
 import threading
 from typing import TYPE_CHECKING, Literal
 
 import numpy as np
+import pyfftw
 
 from .acore import AGenerator
 from .io import Output
@@ -16,6 +18,8 @@ if TYPE_CHECKING:
     from .interface import Interface
     from .typing import ABlock, ABuffer
 
+logger = logging.getLogger(__name__)
+
 
 class Player(Generator):
     def __init__(self,
@@ -64,18 +68,6 @@ class Player(Generator):
         else:
             return self._aplayer.finished_event.is_set()
 
-    def restart(self) -> bool:
-        """Restart the Player, unset the finished() flag.
-
-        Returns:
-            `True`, when successful. `False`, when the Player isn't finished yet.
-        """
-        if not self.finished(block=False):
-            return False
-        self._afile.seek(0)
-        self._aplayer.finished_event.clear()
-        return True
-
     def looped(self, block: bool = True, timeout: float = 1) -> bool:
         """This function can be used to wait for the Player to loop.
 
@@ -106,7 +98,7 @@ class Player(Generator):
 
     class _APlayer(AGenerator):
         def __init__(self,
-                     afile: "AFile",
+                     afile: "AFile|SoundFile",
                      loop: bool,
                      blocksize: int,
                      start_frame: int) -> None:
@@ -117,16 +109,12 @@ class Player(Generator):
             self._afile = afile
             self._loop = loop
             self._samples = afile.frames
-
-            self._firstread = False
             super().__init__(out_buffer=True,  # The _out_buf is used to write the file into on _inc()
                              blocksize=blocksize,
                              start_frame=start_frame)
 
         def _mod(self, outdata: "ABlock", ch: int) -> None:
-            if not self._firstread:
-                self._inc()
-                self._firstread = True
+            return super()._mod(outdata, ch)
 
         def _inc(self) -> None:
             if self.finished_event.is_set():
@@ -149,6 +137,16 @@ class Player(Generator):
             # load a block of samples in the buffer
             self._afile.read(self._blocksize, dtype="float32", always_2d=True, fill_value=0, out=self._out_buf)
 
+        def _reset(self) -> None:
+            self._afile.seek(0)
+            self.looped_event.clear()
+            self.finished_event.clear()
+            while not self.seek_queue.empty():
+                self.seek_queue.get(block=False)
+            # load the first frame into _out_buffer
+            self._inc()
+            return super()._reset()
+
 
 class Sine(Generator):
     def __init__(self,
@@ -197,10 +195,10 @@ class Sine(Generator):
             self._frequency = frequency
             self._samplerate = samplerate
             self._phase = phase
+            self._start_phase = phase
             self._omega_per_block = 2 * np.pi * frequency * blocksize / samplerate
 
-            self._omegas = np.linspace(start_frame * self._omega_per_block,
-                                       (start_frame + 1) * self._omega_per_block,
+            self._omegas = np.linspace(0, self._omega_per_block,
                                        blocksize, endpoint=False, dtype=np.float32)
             super().__init__(out_buffer=out_buffer,
                              blocksize=blocksize,
@@ -219,6 +217,12 @@ class Sine(Generator):
                 self._omegas = np.linspace(0, self._omega_per_block, self._blocksize,
                                            endpoint=False, dtype=np.float32)
 
+        def _reset(self) -> None:
+            self._phase = self._start_phase
+            while not self.frequency_queue.empty():
+                self.frequency_queue.get(block=False)
+            return super()._reset()
+
 
 class SineBurst(Generator):
     def __init__(self,
@@ -259,11 +263,11 @@ class SineBurst(Generator):
             self._frequency = frequency
             self._samplerate = samplerate
             self._phase = phase
+            self._start_phase = phase
             self._maxang = phase + 2 * np.pi * periods
             self._omega_per_block = 2 * np.pi * frequency * blocksize / samplerate
 
-            self._omegas = np.linspace(start_frame * self._omega_per_block,
-                                       (start_frame + 1) * self._omega_per_block,
+            self._omegas = np.linspace(0, self._omega_per_block,
                                        blocksize, endpoint=False, dtype=np.float32)
             super().__init__(out_buffer=out_buffer,
                              blocksize=blocksize,
@@ -278,6 +282,12 @@ class SineBurst(Generator):
             if self._phase < self._maxang:
                 self._phase += self._omega_per_block
 
+        def _reset(self) -> None:
+            self._phase = self._start_phase
+            while not self.frequency_queue.empty():
+                self.frequency_queue.get(block=False)
+            return super()._reset()
+
 
 class Vector(Generator):
     def __init__(self, interface: "Interface", vector: "ABuffer") -> None:
@@ -312,18 +322,6 @@ class Vector(Generator):
         else:
             return self._avector.finished_event.is_set()
 
-    def restart(self) -> bool:
-        """Restart the Player, unset the finished() flag.
-
-        Returns:
-            `True`, when successful. `False`, when the Player isn't finished yet.
-        """
-        if not self.finished(block=False):
-            return False
-        self._avector.pos = 0
-        self._avector.finished_event.clear()
-        return True
-
     class _AVector(AGenerator):
         def __init__(self,
                      vector: "ABuffer",
@@ -332,7 +330,7 @@ class Vector(Generator):
             self.finished_event = threading.Event()
             self._vector = vector
             self._samples = np.size(vector[:, 0])
-            self.pos = 0
+            self._pos = 0
 
             super().__init__(out_buffer=False,
                              blocksize=blocksize,
@@ -341,38 +339,41 @@ class Vector(Generator):
         def _mod(self, outdata: "ABlock", ch: int) -> None:
             if self.finished_event.is_set():
                 return
-            rest = self._samples - self.pos
+            rest = self._samples - self._pos
             if rest < self._blocksize:
                 # read rest (not full vlock)
-                outdata[:rest] = self._vector[:, ch]
+                outdata[:rest] = self._vector[self._pos:, ch]
                 return
-            outdata[:] = self._vector[:, ch]
+            outdata[:] = self._vector[self._pos:self._pos+self._blocksize, ch]
 
         def _inc(self) -> None:
             if self.finished_event.is_set():
                 return
-            self.pos += self._blocksize
+            self._pos += self._blocksize
             # if there are no samples left to play
-            if self._samples - self.pos <= 0:
+            if self._samples - self._pos <= 0:
                 self.finished_event.set()
                 return
 
+        def _reset(self) -> None:
+            self._pos = 0
+            self.finished_event.clear()
+            return super()._reset()
+
 
 class Noise(Generator):
     def __init__(self,
                  interface: "Interface",
-                 weight: Literal["white", "pink"] = "white",
-                 out_buffer: bool = False) -> None:
-        """The WhiteNoise class generator is used to craete a random unweighted noise signal.
+                 weight: Literal["white", "pink"] = "white") -> None:
+        """The WhiteNoise class generator is used to craete a uniform distributed noise signal.
         It is a single output generator, therefore output-update is not supported.
 
         Args:
             interface: Reference to an Interface instance.
             weight: The frequency spectrum of the noise.
-            out_buffer: Flag that decides if outputs are buffered.
         """
 
-        self._asine = self._ANoise(weight, out_buffer, interface.blocksize, interface.start_frame)
+        self._asine = self._ANoise(weight, True, interface.blocksize, interface.samplerate, interface.start_frame)
         super().__init__(agenerator=self._asine,
                          interface=interface,
                          outputs=(Output(self), ),
@@ -383,33 +384,57 @@ class Noise(Generator):
                      weight: Literal["white", "pink"],
                      out_buffer: bool,
                      blocksize: int,
+                     samplerate: int,
                      start_frame: int) -> None:
-            self._tmp = np.empty(blocksize, dtype=np.float64)
+            # setup fft
+            self._tmp = pyfftw.empty_aligned(blocksize, dtype=np.float32)
+            self._rnd_spec = pyfftw.empty_aligned(int(blocksize / 2 + 1), dtype=np.complex64)
+            self._fftw = pyfftw.FFTW(self._tmp,
+                                     self._rnd_spec,
+                                     axes=(0, ))
+            self._ifftw = pyfftw.FFTW(self._rnd_spec,
+                                      self._tmp,
+                                      axes=(0, ),
+                                      direction="FFTW_BACKWARD")
+
             self._rng = np.random.default_rng()
             # weighting
-            self.w = np.ones(int(blocksize / 2 + 1), dtype=np.complex128)
             match(weight):
                 case "pink":
-                    f = np.fft.rfftfreq(blocksize).astype(np.float32)
-                    # the reshape(-1) should do nothing but is needed for
-                    # type-checking
-                    self.w = (1 / np.where(f == 0, float('inf'), np.sqrt(f))).reshape(-1).astype(np.complex128)
+                    logger.warning("Pink noise implementation is resource intensive and "
+                                   "can cause audio issues. Use with caution!")
+                    self._weight = True
+                    f = np.fft.rfftfreq(blocksize, 1/samplerate)
+                    # the reshape(-1) should do nothing but is needed for type-checking
+                    self.w = (1 / np.where(f == 0, float('inf'), np.sqrt(f))).astype(np.complex64)
+                    # normalize window to rms
                     self.w = self.w / np.sqrt(np.mean(self.w**2))
-            # numpy fft only supports complex128
-            self.rnd_spec = np.fft.rfft(np.random.randn(blocksize)).astype(np.complex128)
+                case _:
+                    self._weight = False
+                    self.w = np.ones(int(blocksize / 2 + 1), dtype=np.complex64)
+
             super().__init__(out_buffer=out_buffer,
                              blocksize=blocksize,
                              start_frame=start_frame)
+            # fill first tmp
+            self._inc()
 
         def _mod(self, outdata: "ABlock", ch: int) -> None:
-            np.fft.irfft(self.rnd_spec, out=self._tmp)
-            self._tmp /= 4
-            np.clip(self._tmp, -1, 1, dtype=np.float32, out=outdata)
+            # _mod is only called once anyways because out_buffer = True
+            outdata[:] = self._tmp
 
         def _inc(self) -> None:
-            self._rng.standard_normal(self._blocksize, out=self._tmp)
-            np.fft.rfft(self._tmp, out=self.rnd_spec)
-            self.rnd_spec *= self.w
+            self._rng.random(self._blocksize, dtype=np.float32, out=self._tmp)
+            self._tmp -= 0.5
+            if self._weight:
+                self._fftw.execute()
+                self._rnd_spec *= self.w
+                self._ifftw.execute()
+            norm = np.abs(self._tmp).max()
+            self._tmp /= norm
+
+        def _reset(self) -> None:
+            return super()._reset()
 
 
 class Constant(Generator):
@@ -441,4 +466,7 @@ class Constant(Generator):
             outdata[:] = self._value
 
         def _inc(self) -> None:
-            pass
+            return super()._inc()
+
+        def _reset(self) -> None:
+            return super()._reset()
