@@ -1,10 +1,9 @@
 import logging
-from typing import TYPE_CHECKING, Any, List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Dict, Literal, Optional, Tuple
 
 import sounddevice as sd
 
 from .acore import AInterface
-from .exceptions import DeviceError
 from .io import IInput, IOutput
 from .processor import Processor
 
@@ -12,20 +11,80 @@ if TYPE_CHECKING:
     from pathlib import Path
 
     from .asetup import ASetup
+    from .exceptions import DeviceError
     from .processor import Analyzer
-    from .typing import ADevice, Callback, CData, Device, InAs
+    from .typing import Callback, Device, InAs
 
 logger = logging.getLogger(__name__)
 
 
+class IDevice:
+    def __init__(self, device: Optional[int | str] = None):
+        """Class used internally to manage an audio device.
+
+        Args:
+            device: Numeric device ID or device name substring(s).
+        """
+        # can be initialized via index or str
+        if device is None or (isinstance(device, int) and device < 0):
+            self._index = -1
+            self._name = "NoDevice"
+            self._hostapi = "NoHostApi"
+            return
+        data = sd.query_devices(device)
+        assert isinstance(data, dict)
+        self.deserialize(data)
+
+    def __str__(self) -> str:
+        return f"[{self._index}] {self._name} | {self._hostapi}"
+
+    def __bool__(self) -> bool:
+        return self._index >= 0
+
+    @property
+    def sd(self) -> Optional[int]:
+        """Parse IDevice as sounddevice device."""
+        if self._index >= 0:
+            return self._index
+        return None
+
+    @property
+    def index(self) -> int:
+        return self._index
+
+    @property
+    def hostapi(self) -> str:
+        return self._hostapi
+
+    def deserialize(self, data: Dict[str, Any]) -> None:
+        try:
+            self._index = int(data["index"])
+            self._name = data["name"]
+            if isinstance(data["hostapi"], int):
+                # for int, extract name (because it could differ by system)
+                hostapi = sd.query_hostapis(data["hostapi"])
+                assert isinstance(hostapi, dict)
+                self._hostapi = str(hostapi["name"])
+            elif isinstance(data["hostapi"], str):
+                # for str just store str
+                self._hostapi = data["hostapi"]
+            else:
+                logger.warning("Format of hostapi not recognized.")
+        except KeyError:
+            logger.warning("Not all data given in IDevice deserialize.")
+
+    def serialize(self) -> Dict[str, Any]:
+        return {"index": self._index,
+                "name": self._name,
+                "hostapi": self._hostapi}
+
+
 class Interface(Processor):
     def __init__(self,
                  asetup: Optional["ASetup"] = None,
                  device: "Device" = None,
                  samplerate: int = 44100,
-                 blocksize: int = 1024,
-                 analog_input_channels: Optional[List[int]] = None,
-                 analog_output_channels: Optional[List[int]] = None):
+                 blocksize: int = 1024) -> None:
         """The Interface class represents the audio interface or soundcard.
         It is holding the audio generator and manages settings.
         The settings can either be specified on intialization, by an ASetup class, or used as default.
@@ -38,8 +97,6 @@ class Interface(Processor):
                 If None, the default devices are used.
             samplerate: The samplerate in samples per second.
             blocksize: The blocksize defines the samples per frame.
-            analog_input_channels: List if analog input channels, only works for ASIO, CoreAudio devices.
-            analog_output_channels: List if analog output channels, only works for ASIO, CoreAudio devices.
 
         Notes:
             The device names can be obtained by running
@@ -61,44 +118,29 @@ class Interface(Processor):
             self.latency = 0
 
             # init device if not test
-            self._device: ADevice = (None, None)
-            if not self._is_test():
-                if device is None or device == (None, None):
-                    # if None use default
-                    self._device = sd.default.device
-                elif isinstance(device, str) or isinstance(device, int):
-                    # if single device use for input and output
-                    self._device = (device, device)
-                elif isinstance(device, tuple):
-                    self._device = device
-                else:
-                    DeviceError("Device specification is not correct, refer to the API.")
+            if device is None or device == (None, None):
+                # if None use default
+                default = tuple(sd.default.device)
+                assert len(default) == 2
+                device = default
+                logger.info("No device specified, using default.")
+            # if single device is given, convert to tuple
+            if not isinstance(device, tuple):
+                device = (device, device)
+            # parse device dicts
+            self._device = (IDevice(device[0]), IDevice(device[1]))
 
-            # if not asio or coraaudio or test overwrite the channel lists with all available in-/outputs
-            if not self._is_test() and not self._is_asio() and not self._is_ca():
-                logger.warning("No ASIO or CoreAudio device specified, channel numbers are ignored.")
-                analog_input_channels = list(range(sd.query_devices(device=self._device[0],
-                                                   kind="input")["max_input_channels"]))
-                analog_input_channels = [ch + 1 for ch in analog_input_channels]
-                analog_output_channels = list(range(sd.query_devices(device=self._device[1],
-                                                    kind="output")["max_output_channels"]))
-                analog_output_channels = [ch + 1 for ch in analog_output_channels]
-
-            # set in-/outputs accordingly
-            self._iinputs: Tuple["IInput", ...] = ()
-            if (self._device[0] is not None or self._is_test()) and analog_input_channels is not None:
-                self._iinputs = tuple(IInput(self, iin_ch) for iin_ch in analog_input_channels)
-            self._ioutputs: Tuple["IOutput", ...] = ()
-            if (self._device[1] is not None or self._is_test()) and analog_output_channels is not None:
-                self._ioutputs = tuple(IOutput(self, iout_ch) for iout_ch in analog_output_channels)
+            # init in-/outputs
+            self._iinputs: Tuple[IInput, ...] = ()
+            self._ioutputs: Tuple[IOutput, ...] = ()
 
             self._ainterface = AInterface(blocksize=self._blocksize, start_frame=self.start_frame)
         super().__init__(self)
 
-    def __del__(self) -> None:
-        # deregister from asetup
-        if self._asetup is not None:
-            self._asetup.interface = None
+    @property
+    def callback(self) -> "Callback":
+        """Direct access to the callback function used for testing."""
+        return self._ainterface.callback
 
     @property
     def samplerate(self) -> int:
@@ -108,14 +150,9 @@ class Interface(Processor):
     def blocksize(self) -> int:
         return self._blocksize
 
-    @property
-    def device(self) -> Optional["Device"]:
-        return self._device
-
     @property
     def start_frame(self) -> int:
-        # callback starts at frame 0 so this drops the first three frames
-        return -3
+        return 0
 
     @property
     def asetup(self) -> Optional["ASetup"]:
@@ -147,18 +184,23 @@ class Interface(Processor):
 
     @property
     def iinputs(self) -> Tuple["IInput", ...]:
+        """Returns a list of all IInputs."""
         return self._iinputs
 
     @property
     def ioutputs(self) -> Tuple["IOutput", ...]:
+        """Returns a list of all IOutputs."""
         return self._ioutputs
 
     def iinput(self,
                idx: int = 0,
                ch: Optional[int] = None,
                name: Optional[str] = None,
-               ref: Optional[bool] = None) -> "IInput":
-        """Get the interface analog IInput, which is an asmu Output, by the given argument.
+               ref: Optional[Literal[True]] = None) -> "IInput":
+        """If the given index points to the next element of iinputs list,
+        or if the index exists, but the given channel does not:
+        Create new IInput for given channel and return it.
+        Otherwise, search for the IInput by given arguments in the order ch, name, ref, idx and return it.
 
         Args:
             idx: Index in zero indexed list of IInputs.
@@ -167,21 +209,43 @@ class Interface(Processor):
             ref: The reference IInput.
 
         Raises:
-            ValueError: No IInput for the given argument registered
+            ValueError: Given idx does not point to the next free slot:
+                Cant create more than one channel at once.
+            ValueError: New IInput must specify a physical channel. Use the ch argument.
+            ValueError: No IInput registered for given argument.
 
         Returns:
             Reference to IInput object.
         """
+        # handle idx
+        if idx > len(self._iinputs):
+            raise ValueError("Given idx does not point to the next free slot: "
+                             "Cant create more than one channel at once.")
+        # new iinput, parse other parameters
+        if idx == len(self._iinputs):
+            if ch is None:
+                raise ValueError("New IInput must specify a physical channel. Use the ch argument.")
+            logger.info("Index points to next elemnt in iinputs list: "
+                        "Create new IInput.")
+            iinput = IInput(self, ch, name=name, reference=ref)
+            self._iinputs += (iinput, )
+            return iinput
+        # idx exists, continue and check channel
         if ch is not None:
             try:
                 return next((outpu for outpu in self._iinputs if outpu.channel == ch))
-            except StopIteration as exc:
-                raise ValueError(f"No IInput on channel {ch} registered.") from exc
+            except StopIteration:
+                logger.info("Channel not found, create new IInput.")
+                iinput = IInput(self, ch, name=name, reference=ref)
+                self._iinputs += (iinput, )
+                return iinput
+        # search for name
         if name is not None:
             try:
                 return next((outpu for outpu in self._iinputs if outpu.name == name))
             except StopIteration as exc:
                 raise ValueError(f"No IInput with name {name} registered.") from exc
+        # search for reference
         if ref is not None:
             try:
                 return next((outpu for outpu in self._iinputs if outpu.reference))
@@ -193,8 +257,11 @@ class Interface(Processor):
                 idx: int = 0,
                 ch: Optional[int] = None,
                 name: Optional[str] = None,
-                ref: Optional[bool] = None) -> "IOutput":
-        """Get the interface analog IOutput, which is an asmu Input, by the given argument.
+                ref: Optional[Literal[True]] = None) -> "IOutput":
+        """If the given index points to the next element of ioutputs list,
+        or if the index exists, but the given channel does not:
+        Create new IOutput for given channel and return it.
+        Otherwise, search for the IOutput by given arguments in the order ch, name, ref, idx and return it.
 
         Args:
             idx: Index in zero indexed list of IOutputs.
@@ -203,21 +270,42 @@ class Interface(Processor):
             ref: The reference IInput.
 
         Raises:
-            ValueError: No IOutput for the given argument registered
+            ValueError: Given idx does not point to the next free slot:
+                Cant create more than one channel at once.
+            ValueError: New IInput must specify a physical channel. Use the ch argument.
+            ValueError: No IInput registered for given argument.
 
         Returns:
             Reference to IOutput object.
         """
+        if idx > len(self._ioutputs):
+            raise ValueError("Given idx does not point to the next free slot: "
+                             "Cant create more than one channel at once.")
+        # new iinput, parse other parameters
+        if idx == len(self._ioutputs):
+            if ch is None:
+                raise ValueError("New IOutput must specify a physical channel. Use the ch argument.")
+            logger.info("Index points to next elemnt in iinputs list: "
+                        "Create new IOutput.")
+            ioutput = IOutput(self, ch, name=name, reference=ref)
+            self._ioutputs += (ioutput, )
+            return ioutput
+        # idx exists, continue and check channel
         if ch is not None:
             try:
                 return next((inpu for inpu in self._ioutputs if inpu.channel == ch))
-            except StopIteration as exc:
-                raise ValueError(f"No IOutput on channel {ch} registered.") from exc
+            except StopIteration:
+                logger.info("Channel not found, create new IOutput.")
+                ioutput = IOutput(self, ch, name=name, reference=ref)
+                self._ioutputs += (ioutput, )
+                return ioutput
+        # search for name
         if name is not None:
             try:
                 return next((inpu for inpu in self._ioutputs if inpu.name == name))
             except StopIteration as exc:
                 raise ValueError(f"No IOutput with name {name} registered.") from exc
+        # search for reference
         if ref is not None:
             try:
                 return next((inpu for inpu in self._ioutputs if inpu.reference))
@@ -225,64 +313,72 @@ class Interface(Processor):
                 raise ValueError("No reference IOutput registered.") from exc
         return self._ioutputs[idx]
 
-    def _cal_latency(self, time: "CData") -> None:
-        """Calculate and store loopback latency without physical connection.
+    def get_latency(self) -> int:
+        """Calculate and return loopback latency calculated from buffer times.
 
         !!! warning
             Dont rely on this method, as it only calculates the ADC/DAC's internal latency.
-            Use [latency_from_rec.py](../examples.md/#latency_from_rec.py)
+            Use [calibrate_latency.py](../examples.md/#calibrate_latency.py)
             to compare this result with the real loopback calibration.
 
-        Args:
-            time (CData): The time object given in the callback function.
+        Raises:
+            ValueError: Latency can only be extracted after stream execution.
+            ValueError: Latency computation yielded unplausible values (<1ms).
         """
-        self.latency = round((time.outputBufferDacTime - time.inputBufferAdcTime)
-                             * self.samplerate + 1.0)  # the +1 was measured experimentally (could be the cable?)
+        ctime = self._ainterface.ctime
+        if ctime is None:
+            raise ValueError("Latency can only be extracted after stream execution.")
+        dt = ctime.outputBufferDacTime - ctime.inputBufferAdcTime
+        if dt < 1e-3:
+            raise ValueError("Latency computation yielded unplausible values (<1ms).")
+        return round(dt * self.samplerate + 1.0)  # the +1 was measured experimentally (could be the cable?)
 
-    def _is_asio(self) -> bool:
-        """Determine if ALL of the set io devices are ASIO compatible.
+    def _is_driver(self, drivers: Tuple[str, ...] = ("ASIO", "CoreAudio")) -> bool:
+        """Determine if ALL of the set io devices are one of the given drivers
+        by searching the device name for both driver names.
+        By default we search for ASIO and CoreAudio.
 
         Returns:
-            `True`, when all given devices are ASIO compatible. `False` otherwise.
+            `True`, when all given devices are compatible with one of the given drivers. `False` otherwise.
         """
-        asio = True
-        if self._device[0] is not None:
-            asio = ("asio" in str(sd.query_devices(device=self._device[0], kind="input")["name"]).lower())
-        if self._device[1] is not None:
-            asio = ("asio" in str(sd.query_devices(device=self._device[1], kind="output")["name"]).lower())
-        return asio
+        for driver in drivers:
+            isdriver = True
+            if not self._device[0] and not self._device[1]:
+                return False
+            if self._device[0]:
+                if not (driver.lower() in self._device[0].hostapi.lower()):
+                    isdriver = False
+            if self._device[1]:
+                if not (driver.lower() in self._device[1].hostapi.lower()):
+                    isdriver = False
+            if isdriver:
+                return True
         return False
 
-    def _is_ca(self) -> bool:
-        """Determine if ALL of the set io devices are CoraAudio compatible.
-
-        Returns:
-            `True`, when all given devices are CoraAudio compatible. `False` otherwise.
-        """
-        ca = True
-        if self._device[0] is not None:
-            ca = ("coreaudio" in str(sd.query_devices(device=self._device[0], kind="input")["name"]).lower())
-        if self._device[1] is not None:
-            ca = ("coreaudio" in str(sd.query_devices(device=self._device[1], kind="output")["name"]).lower())
-        return ca
-        return False
-
-    def _is_test(self) -> bool:
-        """Used to skip the sounddevice initialization, if the TestInterface class is derived."""
-        return False
-
-    # SOUNDDEVICE
     def _init_sounddevice(self) -> None:
         """Initiializes sounddevice with the classes attributes for the given lists of inputs and outputs.
+        Depending on the driver, different channel mappings are necessary.
+        ASIO and CoreAudio have internal channel selectors that only use the selected channels in the stream.
+        For other audio frameworks, the stream uses all channels up to the highest needed,
+        and we have to use asmu's channel mapping.
         """
+        # test i/o configuration
+        if not any(self._device):
+            raise DeviceError("Do device specified.")
+        if not self._iinputs and not self._ioutputs:
+            raise ValueError("You should at least specify one input or output channel.")
+        if self._iinputs and not all(self._iinputs):
+            logger.warning("There are unconnected IInputs.")
+        if self._iinputs and not all(self._iinputs):
+            logger.warning("There are unconnected IOutputs")
+
         stream = sd.default
         stream.dtype = ("float32", "float32")
-        if self.samplerate is not None:
-            stream.samplerate = self.samplerate
-        if self.blocksize is not None:
-            stream.blocksize = self.blocksize
-        stream.device = self.device
-        if self._is_asio():
+        stream.samplerate = self.samplerate
+        stream.blocksize = self.blocksize
+        stream.device = (self._device[0].sd, self._device[1].sd)
+
+        if self._is_driver(drivers=("ASIO", )):
             if self._iinputs:
                 # convert to channel names starting with 0
                 in_channels = [inpu.channel - 1 for inpu in self._iinputs]
@@ -307,7 +403,7 @@ class Interface(Processor):
                 stream.channels = (len(in_channels), len(out_channels))
                 return
 
-        elif self._is_ca():
+        if self._is_driver(drivers=("CoreAudio", )):
             raise NotImplementedError("CoreAudio channel selection is not tested!")
             if self._iinputs:
                 # convert to channel names starting with 0
@@ -320,7 +416,7 @@ class Interface(Processor):
                     return
 
             if self._ioutputs:
-                out_channels = [-1] * sd.query_devices(device=self.device, kind="output")["max_output_channels"]
+                out_channels = [-1] * sd.query_devices(device=self._device, kind="output")["max_output_channels"]
                 for idx, c in enumerate(self._ioutputs):
                     out_channels[c.channel - 1] = idx
                 ca_out = sd.CoreAudioSettings(channel_map=out_channels)
@@ -335,11 +431,21 @@ class Interface(Processor):
                 stream.channels = (len(in_channels), len(out_channels))
                 return
 
-    def start(self, end_frame: Optional[int] = None) -> sd.Stream:
+        if self._iinputs and not self._ioutputs:
+            stream.channels = max([inpu.channel for inpu in self._iinputs])
+        if self._ioutputs and not self._iinputs:
+            stream.channels = max([output.channel for output in self._ioutputs])
+        if self._iinputs and self._ioutputs:
+            stream.channels = (max([inpu.channel for inpu in self._iinputs]),
+                               max([output.channel for output in self._ioutputs]))
+
+    def start(self, end_frame: Optional[int] = None, reset: bool = True) -> sd.Stream:
         """Start the audio stream.
 
         Args:
             end_frame: If set, the stream is stopped at the given end_frame.
+            reset: If Truet, all audio processors are reset to initial frame.
+                Otherwise, the Stream continues after the last stop.
 
         Returns:
             Reference to the started sounddevice stream.
@@ -358,6 +464,10 @@ class Interface(Processor):
         """
         self._init_sounddevice()
         self._ainterface.end_frame = end_frame
+        # drop first three frames on reset (used for reset)
+        # also recommended for first start
+        if reset:
+            self._ainterface.frame = self.start_frame - 3
         stream = sd.Stream(callback=self._ainterface.callback)
         stream.start()
         return stream
@@ -373,9 +483,15 @@ class Interface(Processor):
                 # find channel idx it is connected to
                 in_as = in_as + ((inp.connected_output.acore, inp.connected_output.idx), )
         self._ainterface.in_as = in_as
-        # count outputs that have a connection
-        self._ainterface.out_chs = len(self._iinputs)
-        # update alz
+        # update channel maps depending on driver
+        # also see _init_sounddevice for explanation
+        if self._is_driver():
+            self._ainterface.out_ch_map = tuple(range(len(self._iinputs)))
+            self._ainterface.in_ch_map = tuple(range(len(self._ioutputs)))
+        else:
+            self._ainterface.out_ch_map = tuple(inpu.channel - 1 for inpu in self._iinputs)
+            self._ainterface.in_ch_map = tuple(output.channel - 1 for output in self._ioutputs)
+        # update aanalyzers
         self._ainterface.alzs = tuple(alz.acore for alz in self.analyzers)
 
     def serialize(self, setup_path: "Path") -> dict[str, Any]:
@@ -384,8 +500,9 @@ class Interface(Processor):
         data["blocksize"] = int(self._blocksize)
         data["latency"] = int(self.latency)
 
-        if self._device is not None:
-            data["device"] = tuple(self._device)
+        if any(self._device):
+            data["device"] = (self._device[0].serialize(),
+                              self._device[1].serialize())
 
         iinputs = []
         for iinput in self._iinputs:
@@ -403,11 +520,17 @@ class Interface(Processor):
         self._blocksize = int(data["blocksize"])
         self.latency = int(data["latency"])
 
-        self._device = (None, None)
+        self._device = (IDevice(), IDevice())
         try:
-            self._device = tuple(data["device"])
+            dev = tuple(data["device"])
+            self._device[0].deserialize(dev[0])
+            self._device[1].deserialize(dev[1])
         except KeyError:
-            pass
+            # if None use default
+            default = tuple(sd.default.device)
+            assert len(default) == 2
+            self._device = (IDevice(default[0]), IDevice(default[1]))
+            logger.info("No device specified, using default.")
 
         self._iinputs = ()
         for iinput_data in data["iinputs"]:
@@ -422,31 +545,3 @@ class Interface(Processor):
             self._ioutputs += (ioutput, )
 
         self._ainterface = AInterface(blocksize=self._blocksize, start_frame=self.start_frame)
-
-
-class TestInterface(Interface):
-    __test__ = False  # set this so pytest does not collect this class
-
-    def __init__(self,
-                 asetup: Optional["ASetup"] = None,
-                 samplerate: int = 44100,
-                 blocksize: int = 1024,
-                 analog_input_channels: Optional[List[int]] = None,
-                 analog_output_channels: Optional[List[int]] = None) -> None:
-        super().__init__(asetup=asetup,
-                         samplerate=samplerate,
-                         blocksize=blocksize,
-                         analog_input_channels=analog_input_channels,
-                         analog_output_channels=analog_output_channels)
-
-    @property
-    def callback(self) -> "Callback":
-        return self._ainterface.callback
-
-    @property
-    def start_frame(self) -> int:
-        # for testing we want to start directly
-        return 0
-
-    def _is_test(self) -> bool:
-        return True
