@@ -7,9 +7,8 @@ import asmu
 
 def plot_gainramp():
     # create objects
-    interface = asmu.TestInterface(samplerate=44100,
-                                   blocksize=1024,
-                                   analog_output_channels=[1])
+    interface = asmu.Interface(samplerate=44100,
+                               blocksize=1024)
     const = asmu.generator.Constant(interface, 1)
     gainramp = asmu.effect.GainRamp(interface, 0, 0.1, scale="log")
 
@@ -21,7 +20,7 @@ def plot_gainramp():
     fig, ax = plt.subplots()
 
     # setup vectors
-    outdata = np.empty((interface.blocksize, 1), dtype=np.float32)
+    outdata = np.zeros((interface.blocksize, 1), dtype=np.float32)
     x = np.linspace(0, 1, interface.blocksize)
 
     # set gain
@@ -29,7 +28,7 @@ def plot_gainramp():
 
     # run callback loop
     for i in range(12):
-        interface.callback(None, outdata, None, None, None)
+        interface.callback(None, outdata, None, None, None)  # type: ignore[arg-type]
         ax.plot(x + i, outdata, label=f"frame_{i}")
 
     # setup plot and show
