@@ -1,5 +1,9 @@
-"""File that stores useful commands and a small test code for profiling."""
+"""File that stores useful commands and a small test code for profiling.
+To run uncomment the function of interest at the bottom and rund the file with
+`kernprof -l -v ./profiling.py` or
+`python -m memory_profiler ./profiling.py`."""
 import numpy as np
+import plotext
 
 # check for line_profiler or memory_profiler in the local scope, both
 # are injected by their respective tools or they're absent
@@ -12,15 +16,16 @@ if 'prof' not in dir() and 'profile' not in dir():
         return inner
 
 
-@profile
+# @profile
 def copy_test():
     a = np.ones((100000, 2))
-    b = np.empty(100000)
+    b = np.zeros(100000)
 
     np.copyto(b, a[:, 1])
     b[:] = a[:, 1]
 
 
+# @profile
 def slice_test():
     buffer = np.ones((100000, 2))
     data = np.zeros(20000)
@@ -33,16 +38,25 @@ def slice_test():
     buffer[500:20500, 1:2] = data2
 
 
+# @profile
 def random_test():
     buffer = np.zeros(20000)
 
     rng = np.random.default_rng()
+    # normal distribution (slow)
     rng.standard_normal(20000, out=buffer, dtype=np.float64)
-    print(np.max(np.abs(buffer)))
+    # uniform distribution (faster)
+    rng.random(20000, dtype=np.float64, out=buffer)
+    buffer -= 0.5  # center
 
-    buffer = np.random.randn(20000)
+    # normalize (slow)
+    norm = np.max(np.abs(buffer))
+    # normalize (faster)
+    norm = np.abs(buffer).max()
+    buffer /= norm
 
 
+# @profile
 def multiply_test():
     a = np.ones(20000)
     b = np.ones(20000)*5
@@ -54,8 +68,27 @@ def multiply_test():
     a *= b
 
 
-if __name__ == "__main__":
-    copy_test()
+@profile
+def plotext_test():
+    buffer = np.zeros(20000)
+    rng = np.random.default_rng()
+    rng.standard_normal(20000, out=buffer, dtype=np.float64)
 
-# kernprof -l -v ./profiling.py
-# python -m memory_profiler ./profiling.py
+    plotext.title("Time Signal")
+    plotext.grid(True, True)
+    plotext.clc()  # clear color
+
+    # plot
+    for _ in range(10):
+        plotext.cld()
+        plotext.clt()
+        plotext.plot(buffer[::10], marker="braille")
+        plotext.show()
+
+
+if __name__ == "__main__":
+    # copy_test()
+    # slice_test()
+    # random_test()
+    # multiply_test()
+    plotext_test()
