@@ -1,6 +1,4 @@
-"""conftest.py
-Custom PyTest configuration.
-"""
+"""Custom PyTest configuration."""
 import pytest
 
 
